@@ -26,7 +26,7 @@ from winrm.exceptions import (InvalidCredentialsError,
 __author__ = 'Andrey Komissarov'
 __email__ = 'a.komisssarov@gmail.com'
 __date__ = '12.2019'
-__version__ = '1.1.5'
+__version__ = '1.1.6'
 
 logger_name = 'WinOSClient'
 logger = logging.getLogger(logger_name)
@@ -44,8 +44,9 @@ logger.addHandler(ch)
 class ResponseParser:
     """Response parser"""
 
-    def __init__(self, response):
+    def __init__(self, response, command: str = None):
         self.response = response
+        self.command = command
 
     def __repr__(self):
         return str(self.response)
@@ -70,12 +71,12 @@ class ResponseParser:
         except AttributeError:
             stderr = self._decoder(self.response[2])
         err = stderr if stderr else None
-        # if err:
-        #     logger.error(err)
         return err
 
     @property
     def exited(self) -> int:
+        """Get exit code"""
+
         try:
             exited = self.response.status_code
         except AttributeError:
@@ -89,9 +90,15 @@ class ResponseParser:
         except AttributeError:
             return self.response[0] == 0
 
-    def json(self):
+    def json(self) -> dict:
+        """Convert string response into dict"""
         return json.loads(self.stdout)
 
+    @property
+    def cmd(self) -> str:
+        """Show executed command"""
+        return self.command
+
     def decoded(self, encoding: str = 'utf8'):
         """Decode stdout response.
 
@@ -122,6 +129,7 @@ class WinOSClient:
         self.username = username
         self.password = password
         self.logger_enabled = logger_enabled
+        self.logger = logger
 
         logger.disabled = not self.logger_enabled
 
@@ -181,13 +189,12 @@ class WinOSClient:
 
         session = self.session
 
-        protocol = Protocol(
-            endpoint=endpoint,
-            transport=transport,
-            username=self.username,
-            password=self.password,
-            server_cert_validation='ignore',
-            message_encryption='always')
+        protocol = Protocol(endpoint=endpoint,
+                            transport=transport,
+                            username=self.username,
+                            password=self.password,
+                            server_cert_validation='ignore',
+                            message_encryption='always')
 
         session.protocol = protocol
         return session
@@ -206,7 +213,7 @@ class WinOSClient:
         response = None
 
         try:
-            logger.info(f'[{self.host}] ' + command)
+            logger.info(f'[{self.host}] {command}')
             if ps:  # Use PowerShell
                 endpoint = (f'https://{self.host}:5986/wsman'
                             if use_cred_ssp
@@ -219,19 +226,19 @@ class WinOSClient:
                     endpoint=f'http://{self.host}:5985/wsman',
                     transport='ntlm')
                 response = client.run_cmd(command, [arg for arg in args])
-            return ResponseParser(response)
+            return ResponseParser(response, command=command)
 
         # Catch exceptions
         except InvalidCredentialsError as err:
             logger.error(f'Invalid credentials: {self.username}@{self.password}. {err}')
             raise InvalidCredentialsError
         except ConnectionError as err:
-            logger.error('Connection error: ' + str(err))
+            logger.error(f'Connection error: {err}')
             raise ConnectionError
         except (WinRMError,
                 WinRMOperationTimeoutError,
                 WinRMTransportError) as err:
-            logger.error('WinRM error: ' + str(err))
+            logger.error(f'WinRM error: {err}')
             raise err
         except Exception as err:
             logger.error(f'Unhandled error: {err}. Try to use "run_cmd_local" method instead.')
@@ -250,15 +257,15 @@ class WinOSClient:
 
         with Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE) as process:
             try:
-                logger.info('[LOCAL] ' + cmd)
+                logger.info(f'[LOCAL] {cmd}')
                 stdout, stderr = process.communicate(timeout=timeout)
                 exitcode = process.wait(timeout=timeout)
                 response = exitcode, stdout, stderr
-                return ResponseParser(response)
+                return ResponseParser(response, command=cmd)
 
             except TimeoutExpired as err:
                 process.kill()
-                logger.error('Timeout exception: ' + str(err))
+                logger.error(f'Timeout exception: {err}')
                 raise err
 
     # ----------------- Main low-level methods ----------------
@@ -332,7 +339,7 @@ class WinOSClient:
         :param ignore_errors: If ignore_errors is set, errors are ignored. Used for local directory and only
         """
 
-        logger.info('[LOCAL] Removing ' + path)
+        logger.info(f'[LOCAL] Removing {path}')
 
         try:
             if os.path.isfile(path):
@@ -377,12 +384,6 @@ class WinOSClient:
 
         return platform.system()
 
-    @property
-    def is_windows(self):
-        """Is current OS Windows"""
-
-        return 'Windows' in self.get_os_name()
-
     def ping(self, host: str = '', packets_number: int = 4):
         """Ping remote host from current one.
 
@@ -390,10 +391,8 @@ class WinOSClient:
         :param packets_number: Number of packets. 4 by default
         """
 
-        counter = 'n' if self.is_windows else 'c'
         ip_ = host if host else self.host
-
-        command = f'ping -{counter} {packets_number} {ip_}'
+        command = f'ping -n {packets_number} {ip_}'
         return self._run_local(cmd=command)
 
     def exists(self, path: str) -> bool:
@@ -414,7 +413,7 @@ class WinOSClient:
         :return:
         """
 
-        logger.info('[LOCAL] Exists ' + path)
+        logger.info(f'[LOCAL] Exists {path}')
         return os.path.exists(path)
 
     def get_content(self, path):
@@ -482,9 +481,9 @@ class WinOSClient:
         result = self.run_ps(cmd).stdout
         if result is None:
             logger.info('Nothing found. Dont\'t forget to use wildcards.')
-            return None
+            return []
 
-        result_list = result.split()
+        result_list = result.splitlines()
         if last:
             return result_list[0]
         return result_list
@@ -509,9 +508,9 @@ class WinOSClient:
         if last and not mask:
             return max(entities_list, key=os.path.getmtime)
         if mask and not last:
-            return list(filter(lambda x: mask in x, entities_list))
+            return list(filter(lambda x: mask.lower() in x, entities_list))
         if mask and last:
-            filtered = list(filter(lambda x: mask in x, entities_list))
+            filtered = list(filter(lambda x: mask.lower() in x, entities_list))
             return max(filtered, key=os.path.getmtime)
 
         return entities_list
@@ -526,7 +525,6 @@ class WinOSClient:
 
         cmd = fr"(Get-Item '{path}').VersionInfo.{version}Version"
         return self.run_ps(cmd).stdout
-        # logger.warning(f'File {path} not found.')
 
     def get_file_version_local(self, path: str, version: str = 'Product') -> str:
         """Get local windows file version from file property
@@ -540,7 +538,7 @@ class WinOSClient:
         if exists:
             cmd = fr"(Get-Item '{path}').VersionInfo.{version}Version"
             return self.run_ps_local(cmd).stdout
-        logger.warning(f'File {path} not found.')
+        logger.error(f'File {path} not found.')
 
     def get_file_size(self, path: str) -> int:
         """Get remote windows file size. Returns size in bytes.
@@ -551,7 +549,7 @@ class WinOSClient:
 
         result = self.run_ps(f'(Get-Item "{path}").Length')
         if 'Cannot find path' in result.stderr:
-            logger.error(f'File "{path}" not found.')
+            logger.error(f'File [{path}] not found.')
             return 0
         return int(result.stdout)
 
@@ -566,7 +564,7 @@ class WinOSClient:
         try:
             return os.path.getsize(path)
         except FileNotFoundError as err:
-            logger.error(f'File not found. {err}')
+            logger.error(f'File [{path}] not found. {err}')
             raise err
 
     @staticmethod
@@ -625,10 +623,12 @@ class WinOSClient:
                 hash_.update(data)
             return hash_.hexdigest()
 
-    def _get_xml(self):
+    def get_xml(self):
         # (Select-Xml -Path D:\Test.xml -XPath "/configuration/parameters/logging/LogLevel").Node.Value
         msg = 'Use .run_ps() and "Select-Xml" cmdlet. Select-Xml -Path D:\\Test.xml -XPath "/configuration"'
-        raise NotImplementedError(msg)
+        cmd = ''
+        result = self.run_ps(cmd)
+        return result
 
     @staticmethod
     def get_xml_dom_local(path: str, tag: str = None) -> Element:
@@ -820,21 +820,28 @@ class WinOSClient:
         os.makedirs(path, exist_ok=True)
         return self.exists_local(path)
 
-    def _unzip(self, path: str):
-        """Extract .zip archive to destination folder on remote server
+    def unzip(self, path: str, target_directory: str):
+        """Extract .zip archive to destination folder on remote server.
 
-        :param path:
+        Creates destination folder if it does not exist
+
+        :param path: C:\Archives\Draft[v1].Zip
+        :param target_directory: C:\Reference
         :return:
         """
-        print(path)
-        raise NotImplementedError
+
+        cmd = f'Expand-Archive -Path "{path}" -DestinationPath "{target_directory}"'
+        result = self.run_ps(cmd)
+        return result
 
     @staticmethod
     def unzip_local(path: str, target_directory=None):
         """Extract .zip archive to destination folder
 
         Creates destination folder if it does not exist
+
         :param path: Full path to archive
+        :param target_directory: Full path to archive
         """
 
         directory_to_extract_to = target_directory
@@ -933,7 +940,7 @@ class WinOSClient:
         result = self.run_cmd(f'taskkill -im {name} /f')
         if result.exited == 128:
             logger.info(f'Service [{name}] not found')
-            return True
+            raise FileNotFoundError(f'Service [{name}] not found')
         return result.ok
 
     def kill_process_local(self, name: str) -> bool:
